@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -37,7 +38,8 @@ USA.
 (define compile-file)
 (let ((scm-pathname (lambda (path) (pathname-new-type path "scm")))
       (bin-pathname (lambda (path) (pathname-new-type path "bin")))
-      (ext-pathname (lambda (path) (pathname-new-type path "ext")))
+      (ext-pathname (lambda (path) (pathname-default-type path "ext")))
+      (ext-pathname? (lambda (path) (equal? (pathname-type path) "ext")))
       (com-pathname
        (lambda (path)
 	 (pathname-new-type path (compiler:compiled-code-pathname-type)))))
@@ -93,9 +95,11 @@ USA.
 			  (sf/default-declarations
 			   `((USUAL-INTEGRATIONS
 			      ,@compile-file:override-usual-integrations)
-			     ,@(if (null? dependencies)
-				   '()
-				   `((INTEGRATE-EXTERNAL ,@dependencies))))))
+			     ,@(let ((deps (keep-matching-items
+					    dependencies ext-pathname?)))
+				 (if (null? deps)
+				     '()
+				     `((INTEGRATE-EXTERNAL ,@deps)))))))
 		(sf input-file output-file))))
 	  (if (not compile-file:sf-only?)
 	      (process-file (bin-pathname file)
