@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -458,7 +459,7 @@ USA.
 (define (load-char-into-register type source target)
   (let ((target (target-register-reference target)))
     (cond ((zero? type)
-	   (LAP (MOVZX B ,target ,source)))
+	   (LAP (MOVZX Q ,target B ,source)))
 	  (else
 	   (LAP ,@(load-non-pointer target type 0)
 		(MOV B ,target ,source))))))
