@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -1230,6 +1230,14 @@ USA.
 					   'IGNORE-ERRORS)))
        thunk))))
 
+(define warn-errors?
+  (let ((ok "ok"))
+    (lambda (thunk)
+      (let ((v (ignore-errors (lambda () (thunk) ok))))
+	(cond ((eq? v ok) #f)
+	      ((condition? v) (warn v) #t)
+	      (else (error "Unexpected value:" v)))))))
+
 (define (format-error-message message irritants port)
   (fluid-let ((*unparser-list-depth-limit* 2)
 	      (*unparser-list-breadth-limit* 5))
