@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -879,7 +880,7 @@ USA.
 	   (define (compile-rhs clause vars)
 	     (let ((rhs (cdr clause)))
 	       `(LAMBDA (,@vars PORT EMIT FIFO)
-		  ,@vars PORT EMIT FIFO
+		  (DECLARE (IGNORABLE ,@vars PORT EMIT FIFO))
 		  ,@(map compile-action (except-last-pair rhs))
 		  ,(let ((ns (last rhs)))
 		     (cond ((eq? ns 'DONE)
@@ -1189,7 +1190,7 @@ USA.
 		    params-are-expectation?)
   write-parameters)
 
-#; 
+#;
 (define-header "From"
   ;; parser is completely different -- it's a mail address.
   ...
